#! /usr/bin/python3
# -*- coding: UTF-8 -*-
# 14.09.2014

import tkinter as tk
import io, os
from editor import Editor, askSearch
import my_dialogs
import scripts


class SeekByStringFunctions(object):
    """
    Hier werden die Dateien nacheinander zeilenweise auf Substrings untersucht.
    """
    def __init__(self, master):
        self.searchStr    = ""
        self.replaceStr   = ""
        self.seekCase     = False
        self.seekReplace  = False

        # die zu suchenden Strings etc. abfragen
        askSearch(master, self.getGuiInput, regex = False)

        # Beenden, wenn kein Suchstring vorhanden
        if not self.searchStr:
            return

        if self.seekCase:
            self.searchStr = self.searchStr.upper()
        # den Schalter replace deaktivieren, wenn kein replaceStr vorhanden
        if not self.replaceStr:
            self.seekReplace = False

        self.out = io.StringIO()
        self.cnt = 0

        files, startDir = scripts.extract_files(master)

        for file in files:
            # prüfen, ob es sich um eine Textdatei handelt
            if scripts.check_ext(master, file):
                self.line_walker(os.path.join(startDir, file))

        if self.cnt:
            Editor(master, file = self.out)
        else:
            tk.messagebox.showinfo("Suchen + Ersetzen", "Keine Treffer !")


    def getGuiInput(self, a):
        """
        Beim Beenden des Windows für die Suchparameter wird diese Funktion
        als Callback aufgerufen.
        """
        if a is not None:
            self.searchStr   = a[0]
            self.replaceStr  = a[1]
            self.seekCase    = a[2]
            self.seekReplace = a[4]


    def line_walker(self, filename):
        """
        Durchsucht die Datei 'filename' zeilenweise nach dem searchStr,
        wenn capitals = True, dann wird der searchStr und die zu durchsuchende
        Zeile in Großschreibung umgewandelt.
        Gibt eine Liste aller durchsuchten Dateien aus,
        alle Fundstellen (als ganze Zeilen) werden ausgegeben und die
        genaue Fundstelle im Format "Zeile, Spalte, Zeile, Spalte" ausgegeben.
        """
        substr        = self.searchStr
        replace       = self.replaceStr
        lenSearchStr  = len(substr)
        lenReplaceStr = len(replace)

        match = []
        zeile = 0

        try:
            file = open(filename, 'r', encoding = 'utf-8')
            self.out.write("Datei : {}\n".format( filename))

            for rawline in file:
                zeile += 1
                line = rawline.rstrip()
                if self.seekCase:
                    line = line.upper()

                i = line.find(substr)
                if not self.seekReplace:
                    while i > -1:
                        match.append("{}.{},{}.{}".format(zeile, i, zeile, i + lenSearchStr))
                        i = line.find(substr, i + lenSearchStr)

                else:
                    while i > -1:
                        match.append("{}.{},{}.{}".format(zeile, i, zeile, i + lenSearchStr))
                        line = "{}{}{}".format(line[0:i], self.replaceStr,
                                        line[i+lenSearchStr:])
                        i = line.find(substr, i + lenReplaceStr)

                if match:
                    if not self.seekReplace:
                        self.out.write( "{}{}\n\n".format(rawline, match))
                    else:
                        self.out.write("{}{}\n{}\n\n".format(rawline, line, match))

                    self.cnt += 1
                    match = []
        except UnicodeDecodeError:
            print("Fehler beim Einlesen von {}".format(filename))
        finally:
            file.close()


class SeekByTextObject(object):
    """
    Hier wird das Suchen und Ersetzen von scrolledText() bzw. seinen Methoden
    direkt ausgeführt.
    Diese Version fällt deutlich kürzer aus, und unterstützt Regular Expressions
    """
    def __init__(self, master):
        self.searchStr = ""

        # die zu suchenden Strings etc. abfragen
        askSearch(master, self.getGuiInput,
                    title = master.appName,
                    regex = True)

        # Beenden, wenn kein Suchstring vorhanden
        # -> bei leerer Eingabe oder bei einem Abbruch der Eingabe
        if not self.searchStr:
            return

        files, startDir = scripts.extract_files(master)
        filecnt = len(files)
        cnt = 0

        for file in files:
            cnt += 1

#            print("seekByText : ", file)
            master.focus_set()

            if os.path.isfile(file):
                reply = my_dialogs.yesnocancel(master, master.appName,
                    "Datei # {} von {} Dateien:\n\n{}\n\nDatei durchsuchen ?".\
                    format(cnt, filecnt, file))

                if reply is None:
                    break

                elif reply:
                    # Die Dateierweiterung checken
                    if scripts.check_ext(master, file):
                        Editor(master, file = file,
                            searchStr  = self.searchStr,
                            replaceStr = self.replaceStr,
                            replace    = self.seekReplace,
                            seekCase   = self.seekCase,
                            seekAll    = self.seekAll,
                            regex      = self.regex)


    def getGuiInput(self, a):
        """
        Beim Beenden des Windows für die Suchparameter wird diese Funktion
        als Callback aufgerufen.
        """
        if a is not None:
            self.searchStr  = a[0]
            self.replaceStr = a[1]
            self.seekCase   = a[2]
            self.seekAll    = a[3]
            self.seekReplace= a[4]
            self.regex      = a[5]
