#! /usr/bin/python3
# -*- coding: UTF-8 -*-
# 14.09.2014

import tkinter as tk
from tkinter import filedialog
import os, time, datetime, io
import zipfile as z
import scripts

DEFAULT_ZIP_NAME = "back"

def splitLastPath(path):
    """
    Der Vorgabename für ein Zipfile ist oberste Ebene des Namens des Startverzeichnis.
    Ist der Name "", dann wird der DEFAULT_ZIP_NAME zurückgegeben.
    """
#    print(path, path.rfind('\\'), len(path), path[path.rfind('\\'):])
    path = path[path.rfind('\\') + 1:]
    if not path:
        path = DEFAULT_ZIP_NAME
    return path


def ZipFiles(master):
    """
    Schreibt Dateien in ein ZipFile.
    Die Dateien werden in der Datei master.tmpFile übergeben.
    Ein Dateiname wird aus dem DEFAULT_ZIP_NAME + Datum + Index
    automatisch gebildet.
    Als Zielverzeichnis wird das backDir vorgeschlagen,
    Verzeichnis und Dateinamen kann aber nach Bedarf geändert werden.
    """

    _cwd = os.getcwd()      # Damit das cwd am Ende restauriert werden kann
    pathvar = 0
    zipfile = 0

    def callback(a):
        """
        Holt die Einstellungen aus dem ArchivFileWindow ab
        """
        nonlocal pathvar, zipfile
        pathvar = a[0]
        zipfile = a[1]

    # Holt die Files als Liste und das startDir der Files ab
    files, startDir = scripts.extract_files(master)

    if files:
        # Einen Vorschlag für einen eindeutigen ZipFileNamen erstellen
        # Der besteht aus einem String mit angehängtem Datum + 'zip'
        # -> back140913.zip

        backDir = master.check_backDir()
        d = str(datetime.date.today())
        name = "{}_{}{}{}".format(splitLastPath(startDir), d[2:4], d[5:7], d[8:])
        name = os.path.join(backDir, name)
        zipfile = "{}.zip".format(name)

        # falls der Dateiname schon besteht, dann einen Index anhängen
        if os.path.isfile(zipfile):
            zipfile = scripts.make_fileindex(zipfile)

        # zeigt das Window an und stoppt den Programmablauf,
        # callback liefert die Änderungen vor dem Beenden des Windows zurück 
        ArchivFileWindow(master, zipfile, startDir, callback)

        if not zipfile:
            return

        else:
            if not pathvar:                 # relativer Pfad
                _cwd = os.getcwd()
                os.chdir(startDir)

            try:
                zip = z.ZipFile(zipfile, 'w', compression=z.ZIP_DEFLATED)
                for file in files:
                    if pathvar:
                        zip.write(os.path.join(startDir, file))

                    else:
                        zip.write(file)

            except FileNotFoundError:
                tk.messagebox.showwarning(master.appName + ": Zip-Script", \
                        (('Datei {} wurden nicht gefunden,\n' +\
                        'Die Sicherung ist nicht vollständig !').format(file)))

            finally:
                zip.close()
                tk.messagebox.showwarning(master.appName + " Zip-Script", \
                        '{} Dateien wurden gesichert im Archiv\n\n{}'\
                        .format(len(files), zipfile))

    else:
        tk.messagebox.showwarning(master.appName + ": Zip-Script", \
            "Die Dateiliste war leer,\nes konnten keine Dateien gesichert werden!")

    os.chdir(_cwd)


class ArchivFileWindow(tk.Tk):

    init = 0        # Vorgabe beim Programmstart: Relative Pfade

    def __init__(self, master, zipfile, startDir, callback):
        tk.Toplevel.__init__(self)
        self.protocol('WM_DELETE_WINDOW', self._quit)
        self.title(master.appName + ": Dateien komprimieren")
        self.master = master

        wpos = master.winfo_geometry().split('+')
        self.geometry("+{}+{}".format(int(wpos[1])+ 100, int(wpos[2]) + 150))
        self.resizable(width = None, height = None)
        self.bind('<Escape>', self._quit)

        self.zipfile = tk.StringVar()
        self.zipfile.set(zipfile)
        self.callback = callback


        # Pfade
        self.pathvar = tk.IntVar()
        self.f1 = tk.LabelFrame(self, text = "Dateipfade")
        self.f1.grid(column = 0, columnspan = 3, row = 0, padx = 5,pady = 5, sticky = "nsew")

        tk.Radiobutton(self.f1, variable = self.pathvar, value = 0, anchor = 'w',
            text = "Dateien relativ zu Pfad {} archivieren".format(startDir)).\
            grid(column = 1, columnspan = 2, row = 1, padx = 5, pady = 5, sticky = "nsew")

        tk.Radiobutton(self.f1, variable = self.pathvar, value = 1 , anchor = 'w',
            text = "Dateien mit absolutem Pfad archivieren").\
            grid(column = 1, columnspan = 2, row = 2, padx = 5, pady = 5, sticky ="nsew")


        # Die erste Zeile
        tk.Label(self, text = "BackupDatei:", anchor = 'e', width = 13).\
             grid(column = 0, row = 2, padx = 5, pady = 5, sticky = "nsew")

        tk.Label(self, textvariable = self.zipfile, width = 45, anchor = 'w' ).\
              grid(column = 1, row = 2, padx = 5, pady = 5, sticky = "nsw")

        btn1 = tk.Button(self, text = "...", command = self.onDir, width = 12)
        btn1.grid(column = 2, row = 2, padx = 5, pady = 5, sticky = "nsw")


        btn2 = tk.Button(self, text = "Archivieren", command = self.onArchiv, width = 12)
        btn2.grid(column = 1, row = 4, padx = 5, pady = 5, sticky = "nse")
        btn2.bind('<Return>', self.onArchiv)
        btn2.focus_set()

        btn3 = tk.Button(self, text = "Abbrechen", command = self._quit, width = 12)
        btn3.grid(column = 2, row = 4, padx = 5, pady = 5, sticky = "nsw")
        btn3.bind('<Return>', self._quit)

        self.pathvar.set(ArchivFileWindow.init)

        self.transient(master)
        self.grab_set()
        self.mainloop()
        self.destroy()


    def onArchiv(self, e = None):
        ArchivFileWindow.init = (self.pathvar.get())
        self.callback( (self.pathvar.get(), self.zipfile.get()) )
        self.quit()


    def onDir(self):
        dir = filedialog.asksaveasfilename(defaultextension = '.zip',\
                    title = self.master.appName + ": Zip-Datei schreiben",\
                    initialfile = self.zipfile.get(),\
                    filetypes =[("ZIP", "*.zip")] )
        if dir:
            self.zipfile.set(dir)


    def _quit(self, e = None):
        self.callback((-1, ""))
        self.quit()
